#include "settings.h"

Settings::Settings() {
	emuPath = NULL;
	romPath = NULL;
	progPath = NULL;
}

Settings::~Settings() {
	if(emuPath) delete[] emuPath;
	if(romPath) delete[] romPath;
	//throws an exception! why, maby ShellExecute is holding on to it
	//if(progPath) delete[] progPath; 
}

void Settings::procArgs() {
	char * tCmd, * tPtr, * tPtr2;
	int strlen;

	tCmd = GetCommandLine();
	tPtr = strtok(tCmd+1,"\"");
	if(NULL != tPtr) {
		//truncate exe name
		strlen = sizeof(tPtr);
		tPtr2 = strrchr(tPtr,'\\');
		if(tPtr2)
			strlen = tPtr2 - tPtr;

		progPath = new char[strlen];
		strncpy(progPath,tPtr,strlen);
		progPath[strlen] = '\0';
	}
	//***FixMe, call tPtr = strtok(NULL," "); repeatedly to get rest of args
	FreeEnvironmentStrings(tCmd);
}

bool Settings::init() {
	dspMode = getInt("dspMode",0);
	threedMode = getInt("threedMode",0);
	pallet = getInt("pallet",0);
	fSkip = getInt("fSkip",0);
	scale = getInt("scale",1);
	keyMap = getInt("keyMap",0);

	swap = (getInt("swap",0))?true:false;
	timmerHack = (getInt("timmerHack",0))?true:false;
	vfHack = (getInt("vfHack",0))?true:false;
	debug = (getInt("debug",0))?true:false;
	nothrtl = (getInt("nothrtl",0))?true:false;
	status = (getInt("status",0))?true:false;
	disasm = (getInt("disasm",0))?true:false;
	stdOut = (getInt("stdout",0))?true:false;

	emuPath = getString("emuPath","");
	romPath = getString("romPath","");

	procArgs();

	return true;
}

bool Settings::save_settings() {
	setInt("dspMode",dspMode);
	setInt("threedMode",threedMode);
	setInt("pallet",pallet);
	setInt("fSkip",fSkip);
	setInt("scale",scale);
	setInt("keyMap",keyMap);

	setInt("swap",swap);
	setInt("timmerHack",timmerHack);
	setInt("vfHack",vfHack);
	setInt("debug",debug);
	setInt("nothrtl",nothrtl);
	setInt("status",status);
	setInt("disasm",disasm);
	setInt("stdout",stdOut);

	setString("emuPath",emuPath);
	setString("romPath",romPath);

	return true;
}

char * Settings::getString(char * key, char * init) {
	if(key == NULL) return NULL;
	if(init == NULL) init = "";
	char * tstr = new char[256];
	GetPrivateProfileString("main",key,init,tstr,256,"rboy.ini");
	return tstr;
}
int Settings::getInt(char * key, int init) {
	char * tstr;
	int ret;
	tstr = getString(key,NULL);
	if(tstr == NULL || strlen(tstr)<1) {
		if(tstr) delete[] tstr;
		return init;
	}
	ret = atoi(tstr);
	delete[] tstr;

	return ret;
}

double Settings::getDouble(char * key, double init) {
	char * tstr;
	double ret;

	tstr = getString(key,NULL);
	if(tstr == NULL || strlen(tstr)<1) {
		if(tstr) delete[] tstr;
		return init;
	}
	ret = atof(tstr);
	delete[] tstr;

	return ret;
}

void Settings::setString(char * key, char * val) {
	if(key == NULL) return;
	if(val == NULL) return;

	WritePrivateProfileString("main",key,val,"rboy.ini");
}
void Settings::setInt(char * key, int val) {
	static char tstr[30];
	sprintf(tstr,"%d",val);
	setString(key,tstr);
}
void Settings::setDouble(char * key, double val) {
	static char tstr[30];
	sprintf(tstr,"%f",val);
	setString(key,tstr);
}
